# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import dataclasses
import json  # type: ignore
import re
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import (
    gapic_v1,
    operations_v1,
    path_template,
    rest_helpers,
    rest_streaming,
)
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.cloud.location import locations_pb2  # type: ignore
from google.protobuf import json_format
import grpc  # type: ignore
from requests import __version__ as requests_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore


from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore

from google.cloud.retail_v2alpha.types import import_config
from google.cloud.retail_v2alpha.types import product
from google.cloud.retail_v2alpha.types import product as gcr_product
from google.cloud.retail_v2alpha.types import product_service, purge_config

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .base import ProductServiceTransport

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=requests_version,
)


class ProductServiceRestInterceptor:
    """Interceptor for ProductService.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the ProductServiceRestTransport.

    .. code-block:: python
        class MyCustomProductServiceInterceptor(ProductServiceRestInterceptor):
            def pre_add_fulfillment_places(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_add_fulfillment_places(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_add_local_inventories(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_add_local_inventories(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_create_product(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_product(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_delete_product(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def pre_get_product(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_product(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_import_products(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_import_products(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_products(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_products(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_purge_products(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_purge_products(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_remove_fulfillment_places(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_remove_fulfillment_places(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_remove_local_inventories(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_remove_local_inventories(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_set_inventory(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_set_inventory(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_update_product(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_update_product(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = ProductServiceRestTransport(interceptor=MyCustomProductServiceInterceptor())
        client = ProductServiceClient(transport=transport)


    """

    def pre_add_fulfillment_places(
        self,
        request: product_service.AddFulfillmentPlacesRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.AddFulfillmentPlacesRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for add_fulfillment_places

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_add_fulfillment_places(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for add_fulfillment_places

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_add_local_inventories(
        self,
        request: product_service.AddLocalInventoriesRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.AddLocalInventoriesRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for add_local_inventories

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_add_local_inventories(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for add_local_inventories

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_create_product(
        self,
        request: product_service.CreateProductRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.CreateProductRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for create_product

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_create_product(self, response: gcr_product.Product) -> gcr_product.Product:
        """Post-rpc interceptor for create_product

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_delete_product(
        self,
        request: product_service.DeleteProductRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.DeleteProductRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for delete_product

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def pre_get_product(
        self,
        request: product_service.GetProductRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.GetProductRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for get_product

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_get_product(self, response: product.Product) -> product.Product:
        """Post-rpc interceptor for get_product

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_import_products(
        self,
        request: import_config.ImportProductsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[import_config.ImportProductsRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for import_products

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_import_products(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for import_products

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_list_products(
        self,
        request: product_service.ListProductsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.ListProductsRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for list_products

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_list_products(
        self, response: product_service.ListProductsResponse
    ) -> product_service.ListProductsResponse:
        """Post-rpc interceptor for list_products

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_purge_products(
        self,
        request: purge_config.PurgeProductsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[purge_config.PurgeProductsRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for purge_products

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_purge_products(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for purge_products

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_remove_fulfillment_places(
        self,
        request: product_service.RemoveFulfillmentPlacesRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        product_service.RemoveFulfillmentPlacesRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for remove_fulfillment_places

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_remove_fulfillment_places(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for remove_fulfillment_places

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_remove_local_inventories(
        self,
        request: product_service.RemoveLocalInventoriesRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        product_service.RemoveLocalInventoriesRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for remove_local_inventories

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_remove_local_inventories(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for remove_local_inventories

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_set_inventory(
        self,
        request: product_service.SetInventoryRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.SetInventoryRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for set_inventory

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_set_inventory(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for set_inventory

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_update_product(
        self,
        request: product_service.UpdateProductRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[product_service.UpdateProductRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for update_product

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_update_product(self, response: gcr_product.Product) -> gcr_product.Product:
        """Post-rpc interceptor for update_product

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_get_operation(
        self,
        request: operations_pb2.GetOperationRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[operations_pb2.GetOperationRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for get_operation

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_get_operation(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for get_operation

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response

    def pre_list_operations(
        self,
        request: operations_pb2.ListOperationsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[operations_pb2.ListOperationsRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for list_operations

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ProductService server.
        """
        return request, metadata

    def post_list_operations(
        self, response: operations_pb2.ListOperationsResponse
    ) -> operations_pb2.ListOperationsResponse:
        """Post-rpc interceptor for list_operations

        Override in a subclass to manipulate the response
        after it is returned by the ProductService server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class ProductServiceRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: ProductServiceRestInterceptor


class ProductServiceRestTransport(ProductServiceTransport):
    """REST backend transport for ProductService.

    Service for ingesting [Product][google.cloud.retail.v2alpha.Product]
    information of the customer's website.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1

    """

    def __init__(
        self,
        *,
        host: str = "retail.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[ProductServiceRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'retail.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        maybe_url_match = re.match("^(?P<scheme>http(?:s)?://)?(?P<host>.*)$", host)
        if maybe_url_match is None:
            raise ValueError(
                f"Unexpected hostname structure: {host}"
            )  # pragma: NO COVER

        url_match_items = maybe_url_match.groupdict()

        host = f"{url_scheme}://{host}" if not url_match_items["scheme"] else host

        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        self._operations_client: Optional[operations_v1.AbstractOperationsClient] = None
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or ProductServiceRestInterceptor()
        self._prep_wrapped_messages(client_info)

    @property
    def operations_client(self) -> operations_v1.AbstractOperationsClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Only create a new client if we do not already have one.
        if self._operations_client is None:
            http_options: Dict[str, List[Dict[str, str]]] = {
                "google.longrunning.Operations.GetOperation": [
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/branches/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/branches/*/places/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*/locations/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*/operations/*}",
                    },
                ],
                "google.longrunning.Operations.ListOperations": [
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*/locations/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v2alpha/{name=projects/*}/operations",
                    },
                ],
            }

            rest_transport = operations_v1.OperationsRestTransport(
                host=self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                scopes=self._scopes,
                http_options=http_options,
                path_prefix="v2alpha",
            )

            self._operations_client = operations_v1.AbstractOperationsClient(
                transport=rest_transport
            )

        # Return the client from cache.
        return self._operations_client

    class _AddFulfillmentPlaces(ProductServiceRestStub):
        def __hash__(self):
            return hash("AddFulfillmentPlaces")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.AddFulfillmentPlacesRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the add fulfillment places method over HTTP.

            Args:
                request (~.product_service.AddFulfillmentPlacesRequest):
                    The request object. Request message for
                [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2alpha.ProductService.AddFulfillmentPlaces]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{product=projects/*/locations/*/catalogs/*/branches/*/products/**}:addFulfillmentPlaces",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_add_fulfillment_places(
                request, metadata
            )
            pb_request = product_service.AddFulfillmentPlacesRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_add_fulfillment_places(resp)
            return resp

    class _AddLocalInventories(ProductServiceRestStub):
        def __hash__(self):
            return hash("AddLocalInventories")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.AddLocalInventoriesRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the add local inventories method over HTTP.

            Args:
                request (~.product_service.AddLocalInventoriesRequest):
                    The request object. Request message for
                [ProductService.AddLocalInventories][google.cloud.retail.v2alpha.ProductService.AddLocalInventories]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{product=projects/*/locations/*/catalogs/*/branches/*/products/**}:addLocalInventories",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_add_local_inventories(
                request, metadata
            )
            pb_request = product_service.AddLocalInventoriesRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_add_local_inventories(resp)
            return resp

    class _CreateProduct(ProductServiceRestStub):
        def __hash__(self):
            return hash("CreateProduct")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {
            "productId": "",
        }

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.CreateProductRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> gcr_product.Product:
            r"""Call the create product method over HTTP.

            Args:
                request (~.product_service.CreateProductRequest):
                    The request object. Request message for
                [ProductService.CreateProduct][google.cloud.retail.v2alpha.ProductService.CreateProduct]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.gcr_product.Product:
                    Product captures all metadata
                information of items to be recommended
                or searched.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{parent=projects/*/locations/*/catalogs/*/branches/*}/products",
                    "body": "product",
                },
            ]
            request, metadata = self._interceptor.pre_create_product(request, metadata)
            pb_request = product_service.CreateProductRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = gcr_product.Product()
            pb_resp = gcr_product.Product.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_create_product(resp)
            return resp

    class _DeleteProduct(ProductServiceRestStub):
        def __hash__(self):
            return hash("DeleteProduct")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.DeleteProductRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ):
            r"""Call the delete product method over HTTP.

            Args:
                request (~.product_service.DeleteProductRequest):
                    The request object. Request message for
                [ProductService.DeleteProduct][google.cloud.retail.v2alpha.ProductService.DeleteProduct]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.
            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "delete",
                    "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/branches/*/products/**}",
                },
            ]
            request, metadata = self._interceptor.pre_delete_product(request, metadata)
            pb_request = product_service.DeleteProductRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

    class _GetProduct(ProductServiceRestStub):
        def __hash__(self):
            return hash("GetProduct")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.GetProductRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> product.Product:
            r"""Call the get product method over HTTP.

            Args:
                request (~.product_service.GetProductRequest):
                    The request object. Request message for
                [ProductService.GetProduct][google.cloud.retail.v2alpha.ProductService.GetProduct]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.product.Product:
                    Product captures all metadata
                information of items to be recommended
                or searched.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/branches/*/products/**}",
                },
            ]
            request, metadata = self._interceptor.pre_get_product(request, metadata)
            pb_request = product_service.GetProductRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = product.Product()
            pb_resp = product.Product.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_get_product(resp)
            return resp

    class _ImportProducts(ProductServiceRestStub):
        def __hash__(self):
            return hash("ImportProducts")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: import_config.ImportProductsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the import products method over HTTP.

            Args:
                request (~.import_config.ImportProductsRequest):
                    The request object. Request message for Import methods.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{parent=projects/*/locations/*/catalogs/*/branches/*}/products:import",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_import_products(request, metadata)
            pb_request = import_config.ImportProductsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_import_products(resp)
            return resp

    class _ListProducts(ProductServiceRestStub):
        def __hash__(self):
            return hash("ListProducts")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.ListProductsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> product_service.ListProductsResponse:
            r"""Call the list products method over HTTP.

            Args:
                request (~.product_service.ListProductsRequest):
                    The request object. Request message for
                [ProductService.ListProducts][google.cloud.retail.v2alpha.ProductService.ListProducts]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.product_service.ListProductsResponse:
                    Response message for
                [ProductService.ListProducts][google.cloud.retail.v2alpha.ProductService.ListProducts]
                method.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v2alpha/{parent=projects/*/locations/*/catalogs/*/branches/*}/products",
                },
            ]
            request, metadata = self._interceptor.pre_list_products(request, metadata)
            pb_request = product_service.ListProductsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = product_service.ListProductsResponse()
            pb_resp = product_service.ListProductsResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_list_products(resp)
            return resp

    class _PurgeProducts(ProductServiceRestStub):
        def __hash__(self):
            return hash("PurgeProducts")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: purge_config.PurgeProductsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the purge products method over HTTP.

            Args:
                request (~.purge_config.PurgeProductsRequest):
                    The request object. Request message for PurgeProducts
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{parent=projects/*/locations/*/catalogs/*/branches/*}/products:purge",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_purge_products(request, metadata)
            pb_request = purge_config.PurgeProductsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_purge_products(resp)
            return resp

    class _RemoveFulfillmentPlaces(ProductServiceRestStub):
        def __hash__(self):
            return hash("RemoveFulfillmentPlaces")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.RemoveFulfillmentPlacesRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the remove fulfillment places method over HTTP.

            Args:
                request (~.product_service.RemoveFulfillmentPlacesRequest):
                    The request object. Request message for
                [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2alpha.ProductService.RemoveFulfillmentPlaces]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{product=projects/*/locations/*/catalogs/*/branches/*/products/**}:removeFulfillmentPlaces",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_remove_fulfillment_places(
                request, metadata
            )
            pb_request = product_service.RemoveFulfillmentPlacesRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_remove_fulfillment_places(resp)
            return resp

    class _RemoveLocalInventories(ProductServiceRestStub):
        def __hash__(self):
            return hash("RemoveLocalInventories")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.RemoveLocalInventoriesRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the remove local inventories method over HTTP.

            Args:
                request (~.product_service.RemoveLocalInventoriesRequest):
                    The request object. Request message for
                [ProductService.RemoveLocalInventories][google.cloud.retail.v2alpha.ProductService.RemoveLocalInventories]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{product=projects/*/locations/*/catalogs/*/branches/*/products/**}:removeLocalInventories",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_remove_local_inventories(
                request, metadata
            )
            pb_request = product_service.RemoveLocalInventoriesRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_remove_local_inventories(resp)
            return resp

    class _SetInventory(ProductServiceRestStub):
        def __hash__(self):
            return hash("SetInventory")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.SetInventoryRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the set inventory method over HTTP.

            Args:
                request (~.product_service.SetInventoryRequest):
                    The request object. Request message for
                [ProductService.SetInventory][google.cloud.retail.v2alpha.ProductService.SetInventory]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2alpha/{inventory.name=projects/*/locations/*/catalogs/*/branches/*/products/**}:setInventory",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_set_inventory(request, metadata)
            pb_request = product_service.SetInventoryRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_set_inventory(resp)
            return resp

    class _UpdateProduct(ProductServiceRestStub):
        def __hash__(self):
            return hash("UpdateProduct")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: product_service.UpdateProductRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> gcr_product.Product:
            r"""Call the update product method over HTTP.

            Args:
                request (~.product_service.UpdateProductRequest):
                    The request object. Request message for
                [ProductService.UpdateProduct][google.cloud.retail.v2alpha.ProductService.UpdateProduct]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.gcr_product.Product:
                    Product captures all metadata
                information of items to be recommended
                or searched.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "patch",
                    "uri": "/v2alpha/{product.name=projects/*/locations/*/catalogs/*/branches/*/products/**}",
                    "body": "product",
                },
            ]
            request, metadata = self._interceptor.pre_update_product(request, metadata)
            pb_request = product_service.UpdateProductRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = gcr_product.Product()
            pb_resp = gcr_product.Product.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_update_product(resp)
            return resp

    @property
    def add_fulfillment_places(
        self,
    ) -> Callable[
        [product_service.AddFulfillmentPlacesRequest], operations_pb2.Operation
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._AddFulfillmentPlaces(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def add_local_inventories(
        self,
    ) -> Callable[
        [product_service.AddLocalInventoriesRequest], operations_pb2.Operation
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._AddLocalInventories(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def create_product(
        self,
    ) -> Callable[[product_service.CreateProductRequest], gcr_product.Product]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreateProduct(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def delete_product(
        self,
    ) -> Callable[[product_service.DeleteProductRequest], empty_pb2.Empty]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._DeleteProduct(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_product(
        self,
    ) -> Callable[[product_service.GetProductRequest], product.Product]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetProduct(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def import_products(
        self,
    ) -> Callable[[import_config.ImportProductsRequest], operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ImportProducts(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def list_products(
        self,
    ) -> Callable[
        [product_service.ListProductsRequest], product_service.ListProductsResponse
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListProducts(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def purge_products(
        self,
    ) -> Callable[[purge_config.PurgeProductsRequest], operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._PurgeProducts(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def remove_fulfillment_places(
        self,
    ) -> Callable[
        [product_service.RemoveFulfillmentPlacesRequest], operations_pb2.Operation
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._RemoveFulfillmentPlaces(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def remove_local_inventories(
        self,
    ) -> Callable[
        [product_service.RemoveLocalInventoriesRequest], operations_pb2.Operation
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._RemoveLocalInventories(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def set_inventory(
        self,
    ) -> Callable[[product_service.SetInventoryRequest], operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._SetInventory(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def update_product(
        self,
    ) -> Callable[[product_service.UpdateProductRequest], gcr_product.Product]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._UpdateProduct(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_operation(self):
        return self._GetOperation(self._session, self._host, self._interceptor)  # type: ignore

    class _GetOperation(ProductServiceRestStub):
        def __call__(
            self,
            request: operations_pb2.GetOperationRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the get operation method over HTTP.

            Args:
                request (operations_pb2.GetOperationRequest):
                    The request object for GetOperation method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                operations_pb2.Operation: Response from GetOperation method.
            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/branches/*/operations/*}",
                },
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/branches/*/places/*/operations/*}",
                },
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*/operations/*}",
                },
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/locations/*/operations/*}",
                },
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/operations/*}",
                },
            ]

            request, metadata = self._interceptor.pre_get_operation(request, metadata)
            request_kwargs = json_format.MessageToDict(request)
            transcoded_request = path_template.transcode(http_options, **request_kwargs)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(json.dumps(transcoded_request["query_params"]))

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"

            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            resp = operations_pb2.Operation()
            resp = json_format.Parse(response.content.decode("utf-8"), resp)
            resp = self._interceptor.post_get_operation(resp)
            return resp

    @property
    def list_operations(self):
        return self._ListOperations(self._session, self._host, self._interceptor)  # type: ignore

    class _ListOperations(ProductServiceRestStub):
        def __call__(
            self,
            request: operations_pb2.ListOperationsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.ListOperationsResponse:
            r"""Call the list operations method over HTTP.

            Args:
                request (operations_pb2.ListOperationsRequest):
                    The request object for ListOperations method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                operations_pb2.ListOperationsResponse: Response from ListOperations method.
            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/locations/*/catalogs/*}/operations",
                },
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*/locations/*}/operations",
                },
                {
                    "method": "get",
                    "uri": "/v2alpha/{name=projects/*}/operations",
                },
            ]

            request, metadata = self._interceptor.pre_list_operations(request, metadata)
            request_kwargs = json_format.MessageToDict(request)
            transcoded_request = path_template.transcode(http_options, **request_kwargs)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(json.dumps(transcoded_request["query_params"]))

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"

            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            resp = operations_pb2.ListOperationsResponse()
            resp = json_format.Parse(response.content.decode("utf-8"), resp)
            resp = self._interceptor.post_list_operations(resp)
            return resp

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("ProductServiceRestTransport",)



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, grpc_helpers_async
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.cloud.location import locations_pb2  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
import grpc  # type: ignore
from grpc.experimental import aio  # type: ignore

from google.cloud.retail_v2alpha.types import catalog
from google.cloud.retail_v2alpha.types import catalog as gcr_catalog
from google.cloud.retail_v2alpha.types import catalog_service

from .base import DEFAULT_CLIENT_INFO, CatalogServiceTransport
from .grpc import CatalogServiceGrpcTransport


class CatalogServiceGrpcAsyncIOTransport(CatalogServiceTransport):
    """gRPC AsyncIO backend transport for CatalogService.

    Service for managing catalog configuration.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "retail.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "retail.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: Optional[aio.Channel] = None,
        api_mtls_endpoint: Optional[str] = None,
        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'retail.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[aio.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def list_catalogs(
        self,
    ) -> Callable[
        [catalog_service.ListCatalogsRequest],
        Awaitable[catalog_service.ListCatalogsResponse],
    ]:
        r"""Return a callable for the list catalogs method over gRPC.

        Lists all the [Catalog][google.cloud.retail.v2alpha.Catalog]s
        associated with the project.

        Returns:
            Callable[[~.ListCatalogsRequest],
                    Awaitable[~.ListCatalogsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_catalogs" not in self._stubs:
            self._stubs["list_catalogs"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/ListCatalogs",
                request_serializer=catalog_service.ListCatalogsRequest.serialize,
                response_deserializer=catalog_service.ListCatalogsResponse.deserialize,
            )
        return self._stubs["list_catalogs"]

    @property
    def update_catalog(
        self,
    ) -> Callable[
        [catalog_service.UpdateCatalogRequest], Awaitable[gcr_catalog.Catalog]
    ]:
        r"""Return a callable for the update catalog method over gRPC.

        Updates the [Catalog][google.cloud.retail.v2alpha.Catalog]s.

        Returns:
            Callable[[~.UpdateCatalogRequest],
                    Awaitable[~.Catalog]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_catalog" not in self._stubs:
            self._stubs["update_catalog"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/UpdateCatalog",
                request_serializer=catalog_service.UpdateCatalogRequest.serialize,
                response_deserializer=gcr_catalog.Catalog.deserialize,
            )
        return self._stubs["update_catalog"]

    @property
    def set_default_branch(
        self,
    ) -> Callable[
        [catalog_service.SetDefaultBranchRequest], Awaitable[empty_pb2.Empty]
    ]:
        r"""Return a callable for the set default branch method over gRPC.

        Set a specified branch id as default branch. API methods such as
        [SearchService.Search][google.cloud.retail.v2alpha.SearchService.Search],
        [ProductService.GetProduct][google.cloud.retail.v2alpha.ProductService.GetProduct],
        [ProductService.ListProducts][google.cloud.retail.v2alpha.ProductService.ListProducts]
        will treat requests using "default_branch" to the actual branch
        id set as default.

        For example, if ``projects/*/locations/*/catalogs/*/branches/1``
        is set as default, setting
        [SearchRequest.branch][google.cloud.retail.v2alpha.SearchRequest.branch]
        to ``projects/*/locations/*/catalogs/*/branches/default_branch``
        is equivalent to setting
        [SearchRequest.branch][google.cloud.retail.v2alpha.SearchRequest.branch]
        to ``projects/*/locations/*/catalogs/*/branches/1``.

        Using multiple branches can be useful when developers would like
        to have a staging branch to test and verify for future usage.
        When it becomes ready, developers switch on the staging branch
        using this API while keeping using
        ``projects/*/locations/*/catalogs/*/branches/default_branch`` as
        [SearchRequest.branch][google.cloud.retail.v2alpha.SearchRequest.branch]
        to route the traffic to this staging branch.

        CAUTION: If you have live predict/search traffic, switching the
        default branch could potentially cause outages if the ID space
        of the new branch is very different from the old one.

        More specifically:

        -  PredictionService will only return product IDs from branch
           {newBranch}.
        -  SearchService will only return product IDs from branch
           {newBranch} (if branch is not explicitly set).
        -  UserEventService will only join events with products from
           branch {newBranch}.

        Returns:
            Callable[[~.SetDefaultBranchRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "set_default_branch" not in self._stubs:
            self._stubs["set_default_branch"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/SetDefaultBranch",
                request_serializer=catalog_service.SetDefaultBranchRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["set_default_branch"]

    @property
    def get_default_branch(
        self,
    ) -> Callable[
        [catalog_service.GetDefaultBranchRequest],
        Awaitable[catalog_service.GetDefaultBranchResponse],
    ]:
        r"""Return a callable for the get default branch method over gRPC.

        Get which branch is currently default branch set by
        [CatalogService.SetDefaultBranch][google.cloud.retail.v2alpha.CatalogService.SetDefaultBranch]
        method under a specified parent catalog.

        Returns:
            Callable[[~.GetDefaultBranchRequest],
                    Awaitable[~.GetDefaultBranchResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_default_branch" not in self._stubs:
            self._stubs["get_default_branch"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/GetDefaultBranch",
                request_serializer=catalog_service.GetDefaultBranchRequest.serialize,
                response_deserializer=catalog_service.GetDefaultBranchResponse.deserialize,
            )
        return self._stubs["get_default_branch"]

    @property
    def get_completion_config(
        self,
    ) -> Callable[
        [catalog_service.GetCompletionConfigRequest],
        Awaitable[catalog.CompletionConfig],
    ]:
        r"""Return a callable for the get completion config method over gRPC.

        Gets a
        [CompletionConfig][google.cloud.retail.v2alpha.CompletionConfig].

        Returns:
            Callable[[~.GetCompletionConfigRequest],
                    Awaitable[~.CompletionConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_completion_config" not in self._stubs:
            self._stubs["get_completion_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/GetCompletionConfig",
                request_serializer=catalog_service.GetCompletionConfigRequest.serialize,
                response_deserializer=catalog.CompletionConfig.deserialize,
            )
        return self._stubs["get_completion_config"]

    @property
    def update_completion_config(
        self,
    ) -> Callable[
        [catalog_service.UpdateCompletionConfigRequest],
        Awaitable[catalog.CompletionConfig],
    ]:
        r"""Return a callable for the update completion config method over gRPC.

        Updates the
        [CompletionConfig][google.cloud.retail.v2alpha.CompletionConfig]s.

        Returns:
            Callable[[~.UpdateCompletionConfigRequest],
                    Awaitable[~.CompletionConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_completion_config" not in self._stubs:
            self._stubs["update_completion_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/UpdateCompletionConfig",
                request_serializer=catalog_service.UpdateCompletionConfigRequest.serialize,
                response_deserializer=catalog.CompletionConfig.deserialize,
            )
        return self._stubs["update_completion_config"]

    @property
    def get_attributes_config(
        self,
    ) -> Callable[
        [catalog_service.GetAttributesConfigRequest],
        Awaitable[catalog.AttributesConfig],
    ]:
        r"""Return a callable for the get attributes config method over gRPC.

        Gets an
        [AttributesConfig][google.cloud.retail.v2alpha.AttributesConfig].

        Returns:
            Callable[[~.GetAttributesConfigRequest],
                    Awaitable[~.AttributesConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_attributes_config" not in self._stubs:
            self._stubs["get_attributes_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/GetAttributesConfig",
                request_serializer=catalog_service.GetAttributesConfigRequest.serialize,
                response_deserializer=catalog.AttributesConfig.deserialize,
            )
        return self._stubs["get_attributes_config"]

    @property
    def update_attributes_config(
        self,
    ) -> Callable[
        [catalog_service.UpdateAttributesConfigRequest],
        Awaitable[catalog.AttributesConfig],
    ]:
        r"""Return a callable for the update attributes config method over gRPC.

        Updates the
        [AttributesConfig][google.cloud.retail.v2alpha.AttributesConfig].

        The catalog attributes in the request will be updated in the
        catalog, or inserted if they do not exist. Existing catalog
        attributes not included in the request will remain unchanged.
        Attributes that are assigned to products, but do not exist at
        the catalog level, are always included in the response. The
        product attribute is assigned default values for missing catalog
        attribute fields, e.g., searchable and dynamic facetable
        options.

        Returns:
            Callable[[~.UpdateAttributesConfigRequest],
                    Awaitable[~.AttributesConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_attributes_config" not in self._stubs:
            self._stubs["update_attributes_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/UpdateAttributesConfig",
                request_serializer=catalog_service.UpdateAttributesConfigRequest.serialize,
                response_deserializer=catalog.AttributesConfig.deserialize,
            )
        return self._stubs["update_attributes_config"]

    @property
    def add_catalog_attribute(
        self,
    ) -> Callable[
        [catalog_service.AddCatalogAttributeRequest],
        Awaitable[catalog.AttributesConfig],
    ]:
        r"""Return a callable for the add catalog attribute method over gRPC.

        Adds the specified
        [CatalogAttribute][google.cloud.retail.v2alpha.CatalogAttribute]
        to the
        [AttributesConfig][google.cloud.retail.v2alpha.AttributesConfig].

        If the
        [CatalogAttribute][google.cloud.retail.v2alpha.CatalogAttribute]
        to add already exists, an ALREADY_EXISTS error is returned.

        Returns:
            Callable[[~.AddCatalogAttributeRequest],
                    Awaitable[~.AttributesConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "add_catalog_attribute" not in self._stubs:
            self._stubs["add_catalog_attribute"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/AddCatalogAttribute",
                request_serializer=catalog_service.AddCatalogAttributeRequest.serialize,
                response_deserializer=catalog.AttributesConfig.deserialize,
            )
        return self._stubs["add_catalog_attribute"]

    @property
    def remove_catalog_attribute(
        self,
    ) -> Callable[
        [catalog_service.RemoveCatalogAttributeRequest],
        Awaitable[catalog.AttributesConfig],
    ]:
        r"""Return a callable for the remove catalog attribute method over gRPC.

        Removes the specified
        [CatalogAttribute][google.cloud.retail.v2alpha.CatalogAttribute]
        from the
        [AttributesConfig][google.cloud.retail.v2alpha.AttributesConfig].

        If the
        [CatalogAttribute][google.cloud.retail.v2alpha.CatalogAttribute]
        to remove does not exist, a NOT_FOUND error is returned.

        Returns:
            Callable[[~.RemoveCatalogAttributeRequest],
                    Awaitable[~.AttributesConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "remove_catalog_attribute" not in self._stubs:
            self._stubs["remove_catalog_attribute"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/RemoveCatalogAttribute",
                request_serializer=catalog_service.RemoveCatalogAttributeRequest.serialize,
                response_deserializer=catalog.AttributesConfig.deserialize,
            )
        return self._stubs["remove_catalog_attribute"]

    @property
    def batch_remove_catalog_attributes(
        self,
    ) -> Callable[
        [catalog_service.BatchRemoveCatalogAttributesRequest],
        Awaitable[catalog_service.BatchRemoveCatalogAttributesResponse],
    ]:
        r"""Return a callable for the batch remove catalog
        attributes method over gRPC.

        Removes all specified
        [CatalogAttribute][google.cloud.retail.v2alpha.CatalogAttribute]s
        from the
        [AttributesConfig][google.cloud.retail.v2alpha.AttributesConfig].

        Returns:
            Callable[[~.BatchRemoveCatalogAttributesRequest],
                    Awaitable[~.BatchRemoveCatalogAttributesResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "batch_remove_catalog_attributes" not in self._stubs:
            self._stubs[
                "batch_remove_catalog_attributes"
            ] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/BatchRemoveCatalogAttributes",
                request_serializer=catalog_service.BatchRemoveCatalogAttributesRequest.serialize,
                response_deserializer=catalog_service.BatchRemoveCatalogAttributesResponse.deserialize,
            )
        return self._stubs["batch_remove_catalog_attributes"]

    @property
    def replace_catalog_attribute(
        self,
    ) -> Callable[
        [catalog_service.ReplaceCatalogAttributeRequest],
        Awaitable[catalog.AttributesConfig],
    ]:
        r"""Return a callable for the replace catalog attribute method over gRPC.

        Replaces the specified
        [CatalogAttribute][google.cloud.retail.v2alpha.CatalogAttribute]
        in the
        [AttributesConfig][google.cloud.retail.v2alpha.AttributesConfig]
        by updating the catalog attribute with the same
        [CatalogAttribute.key][google.cloud.retail.v2alpha.CatalogAttribute.key].

        If the
        [CatalogAttribute][google.cloud.retail.v2alpha.CatalogAttribute]
        to replace does not exist, a NOT_FOUND error is returned.

        Returns:
            Callable[[~.ReplaceCatalogAttributeRequest],
                    Awaitable[~.AttributesConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "replace_catalog_attribute" not in self._stubs:
            self._stubs["replace_catalog_attribute"] = self.grpc_channel.unary_unary(
                "/google.cloud.retail.v2alpha.CatalogService/ReplaceCatalogAttribute",
                request_serializer=catalog_service.ReplaceCatalogAttributeRequest.serialize,
                response_deserializer=catalog.AttributesConfig.deserialize,
            )
        return self._stubs["replace_catalog_attribute"]

    def close(self):
        return self.grpc_channel.close()

    @property
    def get_operation(
        self,
    ) -> Callable[[operations_pb2.GetOperationRequest], operations_pb2.Operation]:
        r"""Return a callable for the get_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_operation" not in self._stubs:
            self._stubs["get_operation"] = self.grpc_channel.unary_unary(
                "/google.longrunning.Operations/GetOperation",
                request_serializer=operations_pb2.GetOperationRequest.SerializeToString,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["get_operation"]

    @property
    def list_operations(
        self,
    ) -> Callable[
        [operations_pb2.ListOperationsRequest], operations_pb2.ListOperationsResponse
    ]:
        r"""Return a callable for the list_operations method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_operations" not in self._stubs:
            self._stubs["list_operations"] = self.grpc_channel.unary_unary(
                "/google.longrunning.Operations/ListOperations",
                request_serializer=operations_pb2.ListOperationsRequest.SerializeToString,
                response_deserializer=operations_pb2.ListOperationsResponse.FromString,
            )
        return self._stubs["list_operations"]


__all__ = ("CatalogServiceGrpcAsyncIOTransport",)



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.retail_v2.types import common, search_service

__protobuf__ = proto.module(
    package="google.cloud.retail.v2",
    manifest={
        "ServingConfig",
    },
)


class ServingConfig(proto.Message):
    r"""Configures metadata that is used to generate serving time
    results (e.g. search results or recommendation predictions).

    Attributes:
        name (str):
            Immutable. Fully qualified name
            ``projects/*/locations/global/catalogs/*/servingConfig/*``
        display_name (str):
            Required. The human readable serving config display name.
            Used in Retail UI.

            This field must be a UTF-8 encoded string with a length
            limit of 128 characters. Otherwise, an INVALID_ARGUMENT
            error is returned.
        model_id (str):
            The id of the model in the same
            [Catalog][google.cloud.retail.v2.Catalog] to use at serving
            time. Currently only RecommendationModels are supported:
            https://cloud.google.com/retail/recommendations-ai/docs/create-models
            Can be changed but only to a compatible model (e.g.
            others-you-may-like CTR to others-you-may-like CVR).

            Required when
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_RECOMMENDATION][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_RECOMMENDATION].
        price_reranking_level (str):
            How much price ranking we want in serving results. Price
            reranking causes product items with a similar recommendation
            probability to be ordered by price, with the highest-priced
            items first. This setting could result in a decrease in
            click-through and conversion rates. Allowed values are:

            -  ``no-price-reranking``
            -  ``low-price-reranking``
            -  ``medium-price-reranking``
            -  ``high-price-reranking``

            If not specified, we choose default based on model type.
            Default value: ``no-price-reranking``.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_RECOMMENDATION][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_RECOMMENDATION].
        facet_control_ids (MutableSequence[str]):
            Facet specifications for faceted search. If empty, no facets
            are returned. The ids refer to the ids of
            [Control][google.cloud.retail.v2.Control] resources with
            only the Facet control set. These controls are assumed to be
            in the same [Catalog][google.cloud.retail.v2.Catalog] as the
            [ServingConfig][google.cloud.retail.v2.ServingConfig]. A
            maximum of 100 values are allowed. Otherwise, an
            INVALID_ARGUMENT error is returned.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        dynamic_facet_spec (google.cloud.retail_v2.types.SearchRequest.DynamicFacetSpec):
            The specification for dynamically generated facets. Notice
            that only textual facets can be dynamically generated.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        boost_control_ids (MutableSequence[str]):
            Condition boost specifications. If a product matches
            multiple conditions in the specifications, boost scores from
            these specifications are all applied and combined in a
            non-linear way. Maximum number of specifications is 100.

            Notice that if both
            [ServingConfig.boost_control_ids][google.cloud.retail.v2.ServingConfig.boost_control_ids]
            and
            [SearchRequest.boost_spec][google.cloud.retail.v2.SearchRequest.boost_spec]
            are set, the boost conditions from both places are
            evaluated. If a search request matches multiple boost
            conditions, the final boost score is equal to the sum of the
            boost scores from all matched boost conditions.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        filter_control_ids (MutableSequence[str]):
            Condition filter specifications. If a product matches
            multiple conditions in the specifications, filters from
            these specifications are all applied and combined via the
            AND operator. Maximum number of specifications is 100.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        redirect_control_ids (MutableSequence[str]):
            Condition redirect specifications. Only the first triggered
            redirect action is applied, even if multiple apply. Maximum
            number of specifications is 1000.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        twoway_synonyms_control_ids (MutableSequence[str]):
            Condition synonyms specifications. If multiple syonyms
            conditions match, all matching synonyms control in the list
            will execute. Order of controls in the list will not matter.
            Maximum number of specifications is 100.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        oneway_synonyms_control_ids (MutableSequence[str]):
            Condition oneway synonyms specifications. If multiple oneway
            synonyms conditions match, all matching oneway synonyms
            controls in the list will execute. Order of controls in the
            list will not matter. Maximum number of specifications is
            100.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        do_not_associate_control_ids (MutableSequence[str]):
            Condition do not associate specifications. If multiple do
            not associate conditions match, all matching do not
            associate controls in the list will execute.

            -  Order does not matter.
            -  Maximum number of specifications is 100.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        replacement_control_ids (MutableSequence[str]):
            Condition replacement specifications.

            -  Applied according to the order in the list.
            -  A previously replaced term can not be re-replaced.
            -  Maximum number of specifications is 100.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        ignore_control_ids (MutableSequence[str]):
            Condition ignore specifications. If multiple ignore
            conditions match, all matching ignore controls in the list
            will execute.

            -  Order does not matter.
            -  Maximum number of specifications is 100.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].
        diversity_level (str):
            How much diversity to use in recommendation model results
            e.g. ``medium-diversity`` or ``high-diversity``. Currently
            supported values:

            -  ``no-diversity``
            -  ``low-diversity``
            -  ``medium-diversity``
            -  ``high-diversity``
            -  ``auto-diversity``

            If not specified, we choose default based on recommendation
            model type. Default value: ``no-diversity``.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_RECOMMENDATION][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_RECOMMENDATION].
        diversity_type (google.cloud.retail_v2.types.ServingConfig.DiversityType):
            What kind of diversity to use - data driven or rule based.
            If unset, the server behavior defaults to
            [RULE_BASED_DIVERSITY][google.cloud.retail.v2.ServingConfig.DiversityType.RULE_BASED_DIVERSITY].
        enable_category_filter_level (str):
            Whether to add additional category filters on the
            ``similar-items`` model. If not specified, we enable it by
            default. Allowed values are:

            -  ``no-category-match``: No additional filtering of
               original results from the model and the customer's
               filters.
            -  ``relaxed-category-match``: Only keep results with
               categories that match at least one item categories in the
               PredictRequests's context item.

               -  If customer also sends filters in the PredictRequest,
                  then the results will satisfy both conditions (user
                  given and category match).

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_RECOMMENDATION][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_RECOMMENDATION].
        personalization_spec (google.cloud.retail_v2.types.SearchRequest.PersonalizationSpec):
            The specification for personalization spec.

            Can only be set if
            [solution_types][google.cloud.retail.v2.ServingConfig.solution_types]
            is
            [SOLUTION_TYPE_SEARCH][google.cloud.retail.v2main.SolutionType.SOLUTION_TYPE_SEARCH].

            Notice that if both
            [ServingConfig.personalization_spec][google.cloud.retail.v2.ServingConfig.personalization_spec]
            and
            [SearchRequest.personalization_spec][google.cloud.retail.v2.SearchRequest.personalization_spec]
            are set.
            [SearchRequest.personalization_spec][google.cloud.retail.v2.SearchRequest.personalization_spec]
            will override
            [ServingConfig.personalization_spec][google.cloud.retail.v2.ServingConfig.personalization_spec].
        solution_types (MutableSequence[google.cloud.retail_v2.types.SolutionType]):
            Required. Immutable. Specifies the solution
            types that a serving config can be associated
            with. Currently we support setting only one type
            of solution.
    """

    class DiversityType(proto.Enum):
        r"""What type of diversity - data or rule based.

        Values:
            DIVERSITY_TYPE_UNSPECIFIED (0):
                Default value.
            RULE_BASED_DIVERSITY (2):
                Rule based diversity.
            DATA_DRIVEN_DIVERSITY (3):
                Data driven diversity.
        """
        DIVERSITY_TYPE_UNSPECIFIED = 0
        RULE_BASED_DIVERSITY = 2
        DATA_DRIVEN_DIVERSITY = 3

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    model_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    price_reranking_level: str = proto.Field(
        proto.STRING,
        number=4,
    )
    facet_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )
    dynamic_facet_spec: search_service.SearchRequest.DynamicFacetSpec = proto.Field(
        proto.MESSAGE,
        number=6,
        message=search_service.SearchRequest.DynamicFacetSpec,
    )
    boost_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )
    filter_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=9,
    )
    redirect_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=10,
    )
    twoway_synonyms_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=18,
    )
    oneway_synonyms_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=12,
    )
    do_not_associate_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=13,
    )
    replacement_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=14,
    )
    ignore_control_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=15,
    )
    diversity_level: str = proto.Field(
        proto.STRING,
        number=8,
    )
    diversity_type: DiversityType = proto.Field(
        proto.ENUM,
        number=20,
        enum=DiversityType,
    )
    enable_category_filter_level: str = proto.Field(
        proto.STRING,
        number=16,
    )
    personalization_spec: search_service.SearchRequest.PersonalizationSpec = (
        proto.Field(
            proto.MESSAGE,
            number=21,
            message=search_service.SearchRequest.PersonalizationSpec,
        )
    )
    solution_types: MutableSequence[common.SolutionType] = proto.RepeatedField(
        proto.ENUM,
        number=19,
        enum=common.SolutionType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.protobuf import wrappers_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.retail_v2beta.types import common, promotion

__protobuf__ = proto.module(
    package="google.cloud.retail.v2beta",
    manifest={
        "Product",
    },
)


class Product(proto.Message):
    r"""Product captures all metadata information of items to be
    recommended or searched.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        expire_time (google.protobuf.timestamp_pb2.Timestamp):
            The timestamp when this product becomes unavailable for
            [SearchService.Search][google.cloud.retail.v2beta.SearchService.Search].
            Note that this is only applicable to
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            and
            [Type.COLLECTION][google.cloud.retail.v2beta.Product.Type.COLLECTION],
            and ignored for
            [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT].
            In general, we suggest the users to delete the stale
            products explicitly, instead of using this field to
            determine staleness.

            If it is set, the
            [Product][google.cloud.retail.v2beta.Product] is not
            available for
            [SearchService.Search][google.cloud.retail.v2beta.SearchService.Search]
            after
            [expire_time][google.cloud.retail.v2beta.Product.expire_time].
            However, the product can still be retrieved by
            [ProductService.GetProduct][google.cloud.retail.v2beta.ProductService.GetProduct]
            and
            [ProductService.ListProducts][google.cloud.retail.v2beta.ProductService.ListProducts].

            [expire_time][google.cloud.retail.v2beta.Product.expire_time]
            must be later than
            [available_time][google.cloud.retail.v2beta.Product.available_time]
            and
            [publish_time][google.cloud.retail.v2beta.Product.publish_time],
            otherwise an INVALID_ARGUMENT error is thrown.

            Corresponding properties: Google Merchant Center property
            `expiration_date <https://support.google.com/merchants/answer/6324499>`__.

            This field is a member of `oneof`_ ``expiration``.
        ttl (google.protobuf.duration_pb2.Duration):
            Input only. The TTL (time to live) of the product. Note that
            this is only applicable to
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            and
            [Type.COLLECTION][google.cloud.retail.v2beta.Product.Type.COLLECTION],
            and ignored for
            [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT].
            In general, we suggest the users to delete the stale
            products explicitly, instead of using this field to
            determine staleness.

            If it is set, it must be a non-negative value, and
            [expire_time][google.cloud.retail.v2beta.Product.expire_time]
            is set as current timestamp plus
            [ttl][google.cloud.retail.v2beta.Product.ttl]. The derived
            [expire_time][google.cloud.retail.v2beta.Product.expire_time]
            is returned in the output and
            [ttl][google.cloud.retail.v2beta.Product.ttl] is left blank
            when retrieving the
            [Product][google.cloud.retail.v2beta.Product].

            If it is set, the product is not available for
            [SearchService.Search][google.cloud.retail.v2beta.SearchService.Search]
            after current timestamp plus
            [ttl][google.cloud.retail.v2beta.Product.ttl]. However, the
            product can still be retrieved by
            [ProductService.GetProduct][google.cloud.retail.v2beta.ProductService.GetProduct]
            and
            [ProductService.ListProducts][google.cloud.retail.v2beta.ProductService.ListProducts].

            This field is a member of `oneof`_ ``expiration``.
        name (str):
            Immutable. Full resource name of the product, such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/product_id``.
        id (str):
            Immutable. [Product][google.cloud.retail.v2beta.Product]
            identifier, which is the final component of
            [name][google.cloud.retail.v2beta.Product.name]. For
            example, this field is "id_1", if
            [name][google.cloud.retail.v2beta.Product.name] is
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/id_1``.

            This field must be a UTF-8 encoded string with a length
            limit of 128 characters. Otherwise, an INVALID_ARGUMENT
            error is returned.

            Corresponding properties: Google Merchant Center property
            `id <https://support.google.com/merchants/answer/6324405>`__.
            Schema.org property
            `Product.sku <https://schema.org/sku>`__.
        type_ (google.cloud.retail_v2beta.types.Product.Type):
            Immutable. The type of the product. Default to
            [Catalog.product_level_config.ingestion_product_type][google.cloud.retail.v2beta.ProductLevelConfig.ingestion_product_type]
            if unset.
        primary_product_id (str):
            Variant group identifier. Must be an
            [id][google.cloud.retail.v2beta.Product.id], with the same
            parent branch with this product. Otherwise, an error is
            thrown.

            For
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            [Product][google.cloud.retail.v2beta.Product]s, this field
            can only be empty or set to the same value as
            [id][google.cloud.retail.v2beta.Product.id].

            For VARIANT [Product][google.cloud.retail.v2beta.Product]s,
            this field cannot be empty. A maximum of 2,000 products are
            allowed to share the same
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            [Product][google.cloud.retail.v2beta.Product]. Otherwise, an
            INVALID_ARGUMENT error is returned.

            Corresponding properties: Google Merchant Center property
            `item_group_id <https://support.google.com/merchants/answer/6324507>`__.
            Schema.org property
            `Product.inProductGroupWithID <https://schema.org/inProductGroupWithID>`__.
        collection_member_ids (MutableSequence[str]):
            The [id][google.cloud.retail.v2beta.Product.id] of the
            collection members when
            [type][google.cloud.retail.v2beta.Product.type] is
            [Type.COLLECTION][google.cloud.retail.v2beta.Product.Type.COLLECTION].

            Non-existent product ids are allowed. The
            [type][google.cloud.retail.v2beta.Product.type] of the
            members must be either
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            or
            [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT]
            otherwise an INVALID_ARGUMENT error is thrown. Should not
            set it for other types. A maximum of 1000 values are
            allowed. Otherwise, an INVALID_ARGUMENT error is return.
        gtin (str):
            The Global Trade Item Number (GTIN) of the product.

            This field must be a UTF-8 encoded string with a length
            limit of 128 characters. Otherwise, an INVALID_ARGUMENT
            error is returned.

            This field must be a Unigram. Otherwise, an INVALID_ARGUMENT
            error is returned.

            Corresponding properties: Google Merchant Center property
            `gtin <https://support.google.com/merchants/answer/6324461>`__.
            Schema.org property
            `Product.isbn <https://schema.org/isbn>`__,
            `Product.gtin8 <https://schema.org/gtin8>`__,
            `Product.gtin12 <https://schema.org/gtin12>`__,
            `Product.gtin13 <https://schema.org/gtin13>`__, or
            `Product.gtin14 <https://schema.org/gtin14>`__.

            If the value is not a valid GTIN, an INVALID_ARGUMENT error
            is returned.
        categories (MutableSequence[str]):
            Product categories. This field is repeated for supporting
            one product belonging to several parallel categories.
            Strongly recommended using the full path for better search /
            recommendation quality.

            To represent full path of category, use '>' sign to separate
            different hierarchies. If '>' is part of the category name,
            replace it with other character(s).

            For example, if a shoes product belongs to both ["Shoes &
            Accessories" -> "Shoes"] and ["Sports & Fitness" ->
            "Athletic Clothing" -> "Shoes"], it could be represented as:

            ::

                 "categories": [
                   "Shoes & Accessories > Shoes",
                   "Sports & Fitness > Athletic Clothing > Shoes"
                 ]

            Must be set for
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            [Product][google.cloud.retail.v2beta.Product] otherwise an
            INVALID_ARGUMENT error is returned.

            At most 250 values are allowed per
            [Product][google.cloud.retail.v2beta.Product]. Empty values
            are not allowed. Each value must be a UTF-8 encoded string
            with a length limit of 5,000 characters. Otherwise, an
            INVALID_ARGUMENT error is returned.

            Corresponding properties: Google Merchant Center property
            `google_product_category <https://support.google.com/merchants/answer/6324436>`__.
            Schema.org property [Product.category]
            (https://schema.org/category).
        title (str):
            Required. Product title.

            This field must be a UTF-8 encoded string with a length
            limit of 1,000 characters. Otherwise, an INVALID_ARGUMENT
            error is returned.

            Corresponding properties: Google Merchant Center property
            `title <https://support.google.com/merchants/answer/6324415>`__.
            Schema.org property
            `Product.name <https://schema.org/name>`__.
        brands (MutableSequence[str]):
            The brands of the product.

            A maximum of 30 brands are allowed. Each brand must be a
            UTF-8 encoded string with a length limit of 1,000
            characters. Otherwise, an INVALID_ARGUMENT error is
            returned.

            Corresponding properties: Google Merchant Center property
            `brand <https://support.google.com/merchants/answer/6324351>`__.
            Schema.org property
            `Product.brand <https://schema.org/brand>`__.
        description (str):
            Product description.

            This field must be a UTF-8 encoded string with a length
            limit of 5,000 characters. Otherwise, an INVALID_ARGUMENT
            error is returned.

            Corresponding properties: Google Merchant Center property
            `description <https://support.google.com/merchants/answer/6324468>`__.
            Schema.org property
            `Product.description <https://schema.org/description>`__.
        language_code (str):
            Language of the title/description and other string
            attributes. Use language tags defined by `BCP
            47 <https://www.rfc-editor.org/rfc/bcp/bcp47.txt>`__.

            For product prediction, this field is ignored and the model
            automatically detects the text language. The
            [Product][google.cloud.retail.v2beta.Product] can include
            text in different languages, but duplicating
            [Product][google.cloud.retail.v2beta.Product]s to provide
            text in multiple languages can result in degraded model
            performance.

            For product search this field is in use. It defaults to
            "en-US" if unset.
        attributes (MutableMapping[str, google.cloud.retail_v2beta.types.CustomAttribute]):
            Highly encouraged. Extra product attributes to be included.
            For example, for products, this could include the store
            name, vendor, style, color, etc. These are very strong
            signals for recommendation model, thus we highly recommend
            providing the attributes here.

            Features that can take on one of a limited number of
            possible values. Two types of features can be set are:

            Textual features. some examples would be the brand/maker of
            a product, or country of a customer. Numerical features.
            Some examples would be the height/weight of a product, or
            age of a customer.

            For example:
            ``{ "vendor": {"text": ["vendor123", "vendor456"]}, "lengths_cm": {"numbers":[2.3, 15.4]}, "heights_cm": {"numbers":[8.1, 6.4]} }``.

            This field needs to pass all below criteria, otherwise an
            INVALID_ARGUMENT error is returned:

            -  Max entries count: 200.
            -  The key must be a UTF-8 encoded string with a length
               limit of 128 characters.
            -  For indexable attribute, the key must match the pattern:
               ``[a-zA-Z0-9][a-zA-Z0-9_]*``. For example,
               ``key0LikeThis`` or ``KEY_1_LIKE_THIS``.
            -  For text attributes, at most 400 values are allowed.
               Empty values are not allowed. Each value must be a
               non-empty UTF-8 encoded string with a length limit of 256
               characters.
            -  For number attributes, at most 400 values are allowed.
        tags (MutableSequence[str]):
            Custom tags associated with the product.

            At most 250 values are allowed per
            [Product][google.cloud.retail.v2beta.Product]. This value
            must be a UTF-8 encoded string with a length limit of 1,000
            characters. Otherwise, an INVALID_ARGUMENT error is
            returned.

            This tag can be used for filtering recommendation results by
            passing the tag as part of the
            [PredictRequest.filter][google.cloud.retail.v2beta.PredictRequest.filter].

            Corresponding properties: Google Merchant Center property
            `custom_label_0–4 <https://support.google.com/merchants/answer/6324473>`__.
        price_info (google.cloud.retail_v2beta.types.PriceInfo):
            Product price and cost information.

            Corresponding properties: Google Merchant Center property
            `price <https://support.google.com/merchants/answer/6324371>`__.
        rating (google.cloud.retail_v2beta.types.Rating):
            The rating of this product.
        available_time (google.protobuf.timestamp_pb2.Timestamp):
            The timestamp when this
            [Product][google.cloud.retail.v2beta.Product] becomes
            available for
            [SearchService.Search][google.cloud.retail.v2beta.SearchService.Search].
            Note that this is only applicable to
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            and
            [Type.COLLECTION][google.cloud.retail.v2beta.Product.Type.COLLECTION],
            and ignored for
            [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT].
        availability (google.cloud.retail_v2beta.types.Product.Availability):
            The online availability of the
            [Product][google.cloud.retail.v2beta.Product]. Default to
            [Availability.IN_STOCK][google.cloud.retail.v2beta.Product.Availability.IN_STOCK].

            Corresponding properties: Google Merchant Center property
            `availability <https://support.google.com/merchants/answer/6324448>`__.
            Schema.org property
            `Offer.availability <https://schema.org/availability>`__.
        available_quantity (google.protobuf.wrappers_pb2.Int32Value):
            The available quantity of the item.
        fulfillment_info (MutableSequence[google.cloud.retail_v2beta.types.FulfillmentInfo]):
            Fulfillment information, such as the store IDs for in-store
            pickup or region IDs for different shipping methods.

            All the elements must have distinct
            [FulfillmentInfo.type][google.cloud.retail.v2beta.FulfillmentInfo.type].
            Otherwise, an INVALID_ARGUMENT error is returned.
        uri (str):
            Canonical URL directly linking to the product detail page.

            It is strongly recommended to provide a valid uri for the
            product, otherwise the service performance could be
            significantly degraded.

            This field must be a UTF-8 encoded string with a length
            limit of 5,000 characters. Otherwise, an INVALID_ARGUMENT
            error is returned.

            Corresponding properties: Google Merchant Center property
            `link <https://support.google.com/merchants/answer/6324416>`__.
            Schema.org property `Offer.url <https://schema.org/url>`__.
        images (MutableSequence[google.cloud.retail_v2beta.types.Image]):
            Product images for the product. We highly recommend putting
            the main image first.

            A maximum of 300 images are allowed.

            Corresponding properties: Google Merchant Center property
            `image_link <https://support.google.com/merchants/answer/6324350>`__.
            Schema.org property
            `Product.image <https://schema.org/image>`__.
        audience (google.cloud.retail_v2beta.types.Audience):
            The target group associated with a given
            audience (e.g. male, veterans, car owners,
            musicians, etc.) of the product.
        color_info (google.cloud.retail_v2beta.types.ColorInfo):
            The color of the product.

            Corresponding properties: Google Merchant Center property
            `color <https://support.google.com/merchants/answer/6324487>`__.
            Schema.org property
            `Product.color <https://schema.org/color>`__.
        sizes (MutableSequence[str]):
            The size of the product. To represent different size systems
            or size types, consider using this format:
            [[[size_system:]size_type:]size_value].

            For example, in "US:MENS:M", "US" represents size system;
            "MENS" represents size type; "M" represents size value. In
            "GIRLS:27", size system is empty; "GIRLS" represents size
            type; "27" represents size value. In "32 inches", both size
            system and size type are empty, while size value is "32
            inches".

            A maximum of 20 values are allowed per
            [Product][google.cloud.retail.v2beta.Product]. Each value
            must be a UTF-8 encoded string with a length limit of 128
            characters. Otherwise, an INVALID_ARGUMENT error is
            returned.

            Corresponding properties: Google Merchant Center property
            `size <https://support.google.com/merchants/answer/6324492>`__,
            `size_type <https://support.google.com/merchants/answer/6324497>`__,
            and
            `size_system <https://support.google.com/merchants/answer/6324502>`__.
            Schema.org property
            `Product.size <https://schema.org/size>`__.
        materials (MutableSequence[str]):
            The material of the product. For example, "leather",
            "wooden".

            A maximum of 20 values are allowed. Each value must be a
            UTF-8 encoded string with a length limit of 200 characters.
            Otherwise, an INVALID_ARGUMENT error is returned.

            Corresponding properties: Google Merchant Center property
            `material <https://support.google.com/merchants/answer/6324410>`__.
            Schema.org property
            `Product.material <https://schema.org/material>`__.
        patterns (MutableSequence[str]):
            The pattern or graphic print of the product. For example,
            "striped", "polka dot", "paisley".

            A maximum of 20 values are allowed per
            [Product][google.cloud.retail.v2beta.Product]. Each value
            must be a UTF-8 encoded string with a length limit of 128
            characters. Otherwise, an INVALID_ARGUMENT error is
            returned.

            Corresponding properties: Google Merchant Center property
            `pattern <https://support.google.com/merchants/answer/6324483>`__.
            Schema.org property
            `Product.pattern <https://schema.org/pattern>`__.
        conditions (MutableSequence[str]):
            The condition of the product. Strongly encouraged to use the
            standard values: "new", "refurbished", "used".

            A maximum of 1 value is allowed per
            [Product][google.cloud.retail.v2beta.Product]. Each value
            must be a UTF-8 encoded string with a length limit of 128
            characters. Otherwise, an INVALID_ARGUMENT error is
            returned.

            Corresponding properties: Google Merchant Center property
            `condition <https://support.google.com/merchants/answer/6324469>`__.
            Schema.org property
            `Offer.itemCondition <https://schema.org/itemCondition>`__.
        promotions (MutableSequence[google.cloud.retail_v2beta.types.Promotion]):
            The promotions applied to the product. A maximum of 10
            values are allowed per
            [Product][google.cloud.retail.v2beta.Product]. Only
            [Promotion.promotion_id][google.cloud.retail.v2beta.Promotion.promotion_id]
            will be used, other fields will be ignored if set.
        publish_time (google.protobuf.timestamp_pb2.Timestamp):
            The timestamp when the product is published by the retailer
            for the first time, which indicates the freshness of the
            products. Note that this field is different from
            [available_time][google.cloud.retail.v2beta.Product.available_time],
            given it purely describes product freshness regardless of
            when it is available on search and recommendation.
        retrievable_fields (google.protobuf.field_mask_pb2.FieldMask):
            Indicates which fields in the
            [Product][google.cloud.retail.v2beta.Product]s are returned
            in
            [SearchResponse][google.cloud.retail.v2beta.SearchResponse].

            Supported fields for all
            [type][google.cloud.retail.v2beta.Product.type]s:

            -  [audience][google.cloud.retail.v2beta.Product.audience]
            -  [availability][google.cloud.retail.v2beta.Product.availability]
            -  [brands][google.cloud.retail.v2beta.Product.brands]
            -  [color_info][google.cloud.retail.v2beta.Product.color_info]
            -  [conditions][google.cloud.retail.v2beta.Product.conditions]
            -  [gtin][google.cloud.retail.v2beta.Product.gtin]
            -  [materials][google.cloud.retail.v2beta.Product.materials]
            -  [name][google.cloud.retail.v2beta.Product.name]
            -  [patterns][google.cloud.retail.v2beta.Product.patterns]
            -  [price_info][google.cloud.retail.v2beta.Product.price_info]
            -  [rating][google.cloud.retail.v2beta.Product.rating]
            -  [sizes][google.cloud.retail.v2beta.Product.sizes]
            -  [title][google.cloud.retail.v2beta.Product.title]
            -  [uri][google.cloud.retail.v2beta.Product.uri]

            Supported fields only for
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            and
            [Type.COLLECTION][google.cloud.retail.v2beta.Product.Type.COLLECTION]:

            -  [categories][google.cloud.retail.v2beta.Product.categories]
            -  [description][google.cloud.retail.v2beta.Product.description]
            -  [images][google.cloud.retail.v2beta.Product.images]

            Supported fields only for
            [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT]:

            -  Only the first image in
               [images][google.cloud.retail.v2beta.Product.images]

            To mark
            [attributes][google.cloud.retail.v2beta.Product.attributes]
            as retrievable, include paths of the form "attributes.key"
            where "key" is the key of a custom attribute, as specified
            in
            [attributes][google.cloud.retail.v2beta.Product.attributes].

            For
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            and
            [Type.COLLECTION][google.cloud.retail.v2beta.Product.Type.COLLECTION],
            the following fields are always returned in
            [SearchResponse][google.cloud.retail.v2beta.SearchResponse]
            by default:

            -  [name][google.cloud.retail.v2beta.Product.name]

            For
            [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT],
            the following fields are always returned in by default:

            -  [name][google.cloud.retail.v2beta.Product.name]
            -  [color_info][google.cloud.retail.v2beta.Product.color_info]

            The maximum number of paths is 30. Otherwise, an
            INVALID_ARGUMENT error is returned.

            Note: Returning more fields in
            [SearchResponse][google.cloud.retail.v2beta.SearchResponse]
            can increase response payload size and serving latency.

            This field is deprecated. Use the retrievable site-wide
            control instead.
        variants (MutableSequence[google.cloud.retail_v2beta.types.Product]):
            Output only. Product variants grouped together on primary
            product which share similar product attributes. It's
            automatically grouped by
            [primary_product_id][google.cloud.retail.v2beta.Product.primary_product_id]
            for all the product variants. Only populated for
            [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
            [Product][google.cloud.retail.v2beta.Product]s.

            Note: This field is OUTPUT_ONLY for
            [ProductService.GetProduct][google.cloud.retail.v2beta.ProductService.GetProduct].
            Do not set this field in API requests.
        local_inventories (MutableSequence[google.cloud.retail_v2beta.types.LocalInventory]):
            Output only. A list of local inventories specific to
            different places.

            This field can be managed by
            [ProductService.AddLocalInventories][google.cloud.retail.v2beta.ProductService.AddLocalInventories]
            and
            [ProductService.RemoveLocalInventories][google.cloud.retail.v2beta.ProductService.RemoveLocalInventories]
            APIs if fine-grained, high-volume updates are necessary.
    """

    class Type(proto.Enum):
        r"""The type of this product.

        Values:
            TYPE_UNSPECIFIED (0):
                Default value. Default to
                [Catalog.product_level_config.ingestion_product_type][google.cloud.retail.v2beta.ProductLevelConfig.ingestion_product_type]
                if unset.
            PRIMARY (1):
                The primary type.

                As the primary unit for predicting, indexing and search
                serving, a
                [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
                [Product][google.cloud.retail.v2beta.Product] is grouped
                with multiple
                [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT]
                [Product][google.cloud.retail.v2beta.Product]s.
            VARIANT (2):
                The variant type.

                [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT]
                [Product][google.cloud.retail.v2beta.Product]s usually share
                some common attributes on the same
                [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
                [Product][google.cloud.retail.v2beta.Product]s, but they
                have variant attributes like different colors, sizes and
                prices, etc.
            COLLECTION (3):
                The collection type. Collection products are bundled
                [Type.PRIMARY][google.cloud.retail.v2beta.Product.Type.PRIMARY]
                [Product][google.cloud.retail.v2beta.Product]s or
                [Type.VARIANT][google.cloud.retail.v2beta.Product.Type.VARIANT]
                [Product][google.cloud.retail.v2beta.Product]s that are sold
                together, such as a jewelry set with necklaces, earrings and
                rings, etc.
        """
        TYPE_UNSPECIFIED = 0
        PRIMARY = 1
        VARIANT = 2
        COLLECTION = 3

    class Availability(proto.Enum):
        r"""Product availability. If this field is unspecified, the
        product is assumed to be in stock.

        Values:
            AVAILABILITY_UNSPECIFIED (0):
                Default product availability. Default to
                [Availability.IN_STOCK][google.cloud.retail.v2beta.Product.Availability.IN_STOCK]
                if unset.
            IN_STOCK (1):
                Product in stock.
            OUT_OF_STOCK (2):
                Product out of stock.
            PREORDER (3):
                Product that is in pre-order state.
            BACKORDER (4):
                Product that is back-ordered (i.e.
                temporarily out of stock).
        """
        AVAILABILITY_UNSPECIFIED = 0
        IN_STOCK = 1
        OUT_OF_STOCK = 2
        PREORDER = 3
        BACKORDER = 4

    expire_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=16,
        oneof="expiration",
        message=timestamp_pb2.Timestamp,
    )
    ttl: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="expiration",
        message=duration_pb2.Duration,
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    type_: Type = proto.Field(
        proto.ENUM,
        number=3,
        enum=Type,
    )
    primary_product_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    collection_member_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )
    gtin: str = proto.Field(
        proto.STRING,
        number=6,
    )
    categories: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )
    title: str = proto.Field(
        proto.STRING,
        number=8,
    )
    brands: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=9,
    )
    description: str = proto.Field(
        proto.STRING,
        number=10,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=11,
    )
    attributes: MutableMapping[str, common.CustomAttribute] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=12,
        message=common.CustomAttribute,
    )
    tags: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=13,
    )
    price_info: common.PriceInfo = proto.Field(
        proto.MESSAGE,
        number=14,
        message=common.PriceInfo,
    )
    rating: common.Rating = proto.Field(
        proto.MESSAGE,
        number=15,
        message=common.Rating,
    )
    available_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=18,
        message=timestamp_pb2.Timestamp,
    )
    availability: Availability = proto.Field(
        proto.ENUM,
        number=19,
        enum=Availability,
    )
    available_quantity: wrappers_pb2.Int32Value = proto.Field(
        proto.MESSAGE,
        number=20,
        message=wrappers_pb2.Int32Value,
    )
    fulfillment_info: MutableSequence[common.FulfillmentInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=21,
        message=common.FulfillmentInfo,
    )
    uri: str = proto.Field(
        proto.STRING,
        number=22,
    )
    images: MutableSequence[common.Image] = proto.RepeatedField(
        proto.MESSAGE,
        number=23,
        message=common.Image,
    )
    audience: common.Audience = proto.Field(
        proto.MESSAGE,
        number=24,
        message=common.Audience,
    )
    color_info: common.ColorInfo = proto.Field(
        proto.MESSAGE,
        number=25,
        message=common.ColorInfo,
    )
    sizes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=26,
    )
    materials: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=27,
    )
    patterns: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=28,
    )
    conditions: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=29,
    )
    promotions: MutableSequence[promotion.Promotion] = proto.RepeatedField(
        proto.MESSAGE,
        number=34,
        message=promotion.Promotion,
    )
    publish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=33,
        message=timestamp_pb2.Timestamp,
    )
    retrievable_fields: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=30,
        message=field_mask_pb2.FieldMask,
    )
    variants: MutableSequence["Product"] = proto.RepeatedField(
        proto.MESSAGE,
        number=31,
        message="Product",
    )
    local_inventories: MutableSequence[common.LocalInventory] = proto.RepeatedField(
        proto.MESSAGE,
        number=35,
        message=common.LocalInventory,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .catalog import (
    AttributesConfig,
    Catalog,
    CatalogAttribute,
    CompletionConfig,
    MerchantCenterFeedFilter,
    MerchantCenterLink,
    MerchantCenterLinkingConfig,
    ProductLevelConfig,
)
from .catalog_service import (
    AddCatalogAttributeRequest,
    BatchRemoveCatalogAttributesRequest,
    BatchRemoveCatalogAttributesResponse,
    GetAttributesConfigRequest,
    GetCompletionConfigRequest,
    GetDefaultBranchRequest,
    GetDefaultBranchResponse,
    ListCatalogsRequest,
    ListCatalogsResponse,
    RemoveCatalogAttributeRequest,
    ReplaceCatalogAttributeRequest,
    SetDefaultBranchRequest,
    UpdateAttributesConfigRequest,
    UpdateCatalogRequest,
    UpdateCompletionConfigRequest,
)
from .common import (
    AttributeConfigLevel,
    Audience,
    ColorInfo,
    Condition,
    CustomAttribute,
    FulfillmentInfo,
    Image,
    Interval,
    LocalInventory,
    PriceInfo,
    Rating,
    RecommendationsFilteringOption,
    Rule,
    SearchSolutionUseCase,
    SolutionType,
    UserInfo,
)
from .completion_service import CompleteQueryRequest, CompleteQueryResponse
from .control import Control
from .control_service import (
    CreateControlRequest,
    DeleteControlRequest,
    GetControlRequest,
    ListControlsRequest,
    ListControlsResponse,
    UpdateControlRequest,
)
from .export_config import (
    BigQueryOutputResult,
    ExportAnalyticsMetricsRequest,
    ExportAnalyticsMetricsResponse,
    ExportErrorsConfig,
    ExportMetadata,
    ExportProductsResponse,
    ExportUserEventsResponse,
    GcsOutputResult,
    OutputConfig,
    OutputResult,
)
from .import_config import (
    BigQuerySource,
    CompletionDataInputConfig,
    GcsSource,
    ImportCompletionDataRequest,
    ImportCompletionDataResponse,
    ImportErrorsConfig,
    ImportMetadata,
    ImportProductsRequest,
    ImportProductsResponse,
    ImportUserEventsRequest,
    ImportUserEventsResponse,
    ProductInlineSource,
    ProductInputConfig,
    UserEventImportSummary,
    UserEventInlineSource,
    UserEventInputConfig,
)
from .model import Model
from .model_service import (
    CreateModelMetadata,
    CreateModelRequest,
    DeleteModelRequest,
    GetModelRequest,
    ListModelsRequest,
    ListModelsResponse,
    PauseModelRequest,
    ResumeModelRequest,
    TuneModelMetadata,
    TuneModelRequest,
    TuneModelResponse,
    UpdateModelRequest,
)
from .prediction_service import PredictRequest, PredictResponse
from .product import Product
from .product_service import (
    AddFulfillmentPlacesMetadata,
    AddFulfillmentPlacesRequest,
    AddFulfillmentPlacesResponse,
    AddLocalInventoriesMetadata,
    AddLocalInventoriesRequest,
    AddLocalInventoriesResponse,
    CreateProductRequest,
    DeleteProductRequest,
    GetProductRequest,
    ListProductsRequest,
    ListProductsResponse,
    RemoveFulfillmentPlacesMetadata,
    RemoveFulfillmentPlacesRequest,
    RemoveFulfillmentPlacesResponse,
    RemoveLocalInventoriesMetadata,
    RemoveLocalInventoriesRequest,
    RemoveLocalInventoriesResponse,
    SetInventoryMetadata,
    SetInventoryRequest,
    SetInventoryResponse,
    UpdateProductRequest,
)
from .promotion import Promotion
from .purge_config import PurgeMetadata, PurgeUserEventsRequest, PurgeUserEventsResponse
from .search_service import ExperimentInfo, SearchRequest, SearchResponse
from .serving_config import ServingConfig
from .serving_config_service import (
    AddControlRequest,
    CreateServingConfigRequest,
    DeleteServingConfigRequest,
    GetServingConfigRequest,
    ListServingConfigsRequest,
    ListServingConfigsResponse,
    RemoveControlRequest,
    UpdateServingConfigRequest,
)
from .user_event import CompletionDetail, ProductDetail, PurchaseTransaction, UserEvent
from .user_event_service import (
    CollectUserEventRequest,
    RejoinUserEventsMetadata,
    RejoinUserEventsRequest,
    RejoinUserEventsResponse,
    WriteUserEventRequest,
)

__all__ = (
    "AttributesConfig",
    "Catalog",
    "CatalogAttribute",
    "CompletionConfig",
    "MerchantCenterFeedFilter",
    "MerchantCenterLink",
    "MerchantCenterLinkingConfig",
    "ProductLevelConfig",
    "AddCatalogAttributeRequest",
    "BatchRemoveCatalogAttributesRequest",
    "BatchRemoveCatalogAttributesResponse",
    "GetAttributesConfigRequest",
    "GetCompletionConfigRequest",
    "GetDefaultBranchRequest",
    "GetDefaultBranchResponse",
    "ListCatalogsRequest",
    "ListCatalogsResponse",
    "RemoveCatalogAttributeRequest",
    "ReplaceCatalogAttributeRequest",
    "SetDefaultBranchRequest",
    "UpdateAttributesConfigRequest",
    "UpdateCatalogRequest",
    "UpdateCompletionConfigRequest",
    "Audience",
    "ColorInfo",
    "Condition",
    "CustomAttribute",
    "FulfillmentInfo",
    "Image",
    "Interval",
    "LocalInventory",
    "PriceInfo",
    "Rating",
    "Rule",
    "UserInfo",
    "AttributeConfigLevel",
    "RecommendationsFilteringOption",
    "SearchSolutionUseCase",
    "SolutionType",
    "CompleteQueryRequest",
    "CompleteQueryResponse",
    "Control",
    "CreateControlRequest",
    "DeleteControlRequest",
    "GetControlRequest",
    "ListControlsRequest",
    "ListControlsResponse",
    "UpdateControlRequest",
    "BigQueryOutputResult",
    "ExportAnalyticsMetricsRequest",
    "ExportAnalyticsMetricsResponse",
    "ExportErrorsConfig",
    "ExportMetadata",
    "ExportProductsResponse",
    "ExportUserEventsResponse",
    "GcsOutputResult",
    "OutputConfig",
    "OutputResult",
    "BigQuerySource",
    "CompletionDataInputConfig",
    "GcsSource",
    "ImportCompletionDataRequest",
    "ImportCompletionDataResponse",
    "ImportErrorsConfig",
    "ImportMetadata",
    "ImportProductsRequest",
    "ImportProductsResponse",
    "ImportUserEventsRequest",
    "ImportUserEventsResponse",
    "ProductInlineSource",
    "ProductInputConfig",
    "UserEventImportSummary",
    "UserEventInlineSource",
    "UserEventInputConfig",
    "Model",
    "CreateModelMetadata",
    "CreateModelRequest",
    "DeleteModelRequest",
    "GetModelRequest",
    "ListModelsRequest",
    "ListModelsResponse",
    "PauseModelRequest",
    "ResumeModelRequest",
    "TuneModelMetadata",
    "TuneModelRequest",
    "TuneModelResponse",
    "UpdateModelRequest",
    "PredictRequest",
    "PredictResponse",
    "Product",
    "AddFulfillmentPlacesMetadata",
    "AddFulfillmentPlacesRequest",
    "AddFulfillmentPlacesResponse",
    "AddLocalInventoriesMetadata",
    "AddLocalInventoriesRequest",
    "AddLocalInventoriesResponse",
    "CreateProductRequest",
    "DeleteProductRequest",
    "GetProductRequest",
    "ListProductsRequest",
    "ListProductsResponse",
    "RemoveFulfillmentPlacesMetadata",
    "RemoveFulfillmentPlacesRequest",
    "RemoveFulfillmentPlacesResponse",
    "RemoveLocalInventoriesMetadata",
    "RemoveLocalInventoriesRequest",
    "RemoveLocalInventoriesResponse",
    "SetInventoryMetadata",
    "SetInventoryRequest",
    "SetInventoryResponse",
    "UpdateProductRequest",
    "Promotion",
    "PurgeMetadata",
    "PurgeUserEventsRequest",
    "PurgeUserEventsResponse",
    "ExperimentInfo",
    "SearchRequest",
    "SearchResponse",
    "ServingConfig",
    "AddControlRequest",
    "CreateServingConfigRequest",
    "DeleteServingConfigRequest",
    "GetServingConfigRequest",
    "ListServingConfigsRequest",
    "ListServingConfigsResponse",
    "RemoveControlRequest",
    "UpdateServingConfigRequest",
    "CompletionDetail",
    "ProductDetail",
    "PurchaseTransaction",
    "UserEvent",
    "CollectUserEventRequest",
    "RejoinUserEventsMetadata",
    "RejoinUserEventsRequest",
    "RejoinUserEventsResponse",
    "WriteUserEventRequest",
)
